@@ -1,6 +1,6 @@
 /* ELF object file format.
-   Copyright 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002
-   Free Software Foundation, Inc.
+   Copyright 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+   2002, 2003 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -100,7 +100,7 @@ struct elf_obj_sy
 #ifndef obj_begin
 #define obj_begin() elf_begin ()
 #endif
-extern void elf_begin PARAMS ((void));
+extern void elf_begin (void);
 
 /* should be conditional on address size! */
 #define elf_symbol(asymbol) ((elf_symbol_type *) (&(asymbol)->the_bfd))
@@ -123,7 +123,7 @@ extern void elf_begin PARAMS ((void));
   (elf_symbol (symbol_get_bfdsym (S))->internal_elf_sym.st_value = (V))
 #endif
 
-int elf_s_get_other PARAMS ((symbolS *));
+int elf_s_get_other (symbolS *);
 #ifndef S_GET_OTHER
 #define S_GET_OTHER(S)	(elf_s_get_other (S))
 #endif
@@ -134,54 +134,63 @@ int elf_s_get_other PARAMS ((symbolS *));
 
 extern asection *gdb_section;
 
+#ifndef obj_sec_set_private_data
+#define obj_sec_set_private_data(B, S) \
+  if (! BFD_SEND ((B), _new_section_hook, ((B), (S)))) \
+    as_fatal (_("can't allocate ELF private section data: %s"),	\
+	      bfd_errmsg (bfd_get_error ()))
+#endif
+
 #ifndef obj_frob_file
 #define obj_frob_file  elf_frob_file
 #endif
-extern void elf_frob_file PARAMS ((void));
+extern void elf_frob_file (void);
 
 #ifndef obj_frob_file_before_adjust
 #define obj_frob_file_before_adjust  elf_frob_file_before_adjust
 #endif
-extern void elf_frob_file_before_adjust PARAMS ((void));
+extern void elf_frob_file_before_adjust (void);
 
 #ifndef obj_frob_file_after_relocs
 #define obj_frob_file_after_relocs  elf_frob_file_after_relocs
 #endif
-extern void elf_frob_file_after_relocs PARAMS ((void));
+extern void elf_frob_file_after_relocs (void);
 
 #ifndef obj_app_file
 #define obj_app_file elf_file_symbol
 #endif
-extern void elf_file_symbol PARAMS ((const char *));
+extern void elf_file_symbol (const char *, int);
 
-extern void obj_elf_section_change_hook PARAMS ((void));
+extern void obj_elf_section_change_hook (void);
 
-extern void obj_elf_section PARAMS ((int));
-extern void obj_elf_previous PARAMS ((int));
-extern void obj_elf_version PARAMS ((int));
-extern void obj_elf_common PARAMS ((int));
-extern void obj_elf_data PARAMS ((int));
-extern void obj_elf_text PARAMS ((int));
-extern struct fix *obj_elf_vtable_inherit PARAMS ((int));
-extern struct fix *obj_elf_vtable_entry PARAMS ((int));
+extern void obj_elf_section (int);
+extern void obj_elf_previous (int);
+extern void obj_elf_version (int);
+extern void obj_elf_common (int);
+extern void obj_elf_data (int);
+extern void obj_elf_text (int);
+extern void obj_elf_change_section
+  (const char *, int, int, int, const char *, int, int);
+extern struct fix *obj_elf_vtable_inherit (int);
+extern struct fix *obj_elf_vtable_entry (int);
 
 /* BFD wants to write the udata field, which is a no-no for the
-   globally defined sections.  */
+   predefined section symbols in bfd/section.c.  They are read-only.  */
 #ifndef obj_sec_sym_ok_for_reloc
 #define obj_sec_sym_ok_for_reloc(SEC)	((SEC)->owner != 0)
 #endif
 
-void elf_obj_read_begin_hook PARAMS ((void));
+void elf_obj_read_begin_hook (void);
 #ifndef obj_read_begin_hook
 #define obj_read_begin_hook	elf_obj_read_begin_hook
 #endif
 
-void elf_obj_symbol_new_hook PARAMS ((symbolS *));
+void elf_obj_symbol_new_hook (symbolS *);
 #ifndef obj_symbol_new_hook
 #define obj_symbol_new_hook	elf_obj_symbol_new_hook
 #endif
 
-void elf_copy_symbol_attributes PARAMS ((symbolS *, symbolS *));
+void elf_copy_symbol_attributes (symbolS *, symbolS *);
 #ifndef OBJ_COPY_SYMBOL_ATTRIBUTES
 #define OBJ_COPY_SYMBOL_ATTRIBUTES(DEST, SRC) \
   (elf_copy_symbol_attributes (DEST, SRC))
@@ -198,7 +207,7 @@ void elf_copy_symbol_attributes PARAMS ((symbolS *, symbolS *));
 
 /* We need 12 bytes at the start of the section to hold some initial
    information.  */
-extern void obj_elf_init_stab_section PARAMS ((segT));
+extern void obj_elf_init_stab_section (segT);
 #define INIT_STAB_SECTION(seg) obj_elf_init_stab_section (seg)
 
 #ifdef ECOFF_DEBUGGING
@@ -220,12 +229,12 @@ extern void obj_elf_init_stab_section PARAMS ((segT));
 
 #endif /* SEPARATE_STAB_SECTIONS not defined.  */
 
-extern void elf_frob_symbol PARAMS ((symbolS *, int *));
+extern void elf_frob_symbol (symbolS *, int *);
 #ifndef obj_frob_symbol
 #define obj_frob_symbol(symp, punt) elf_frob_symbol (symp, &punt)
 #endif
 
-extern void elf_pop_insert PARAMS ((void));
+extern void elf_pop_insert (void);
 #ifndef obj_pop_insert
 #define obj_pop_insert()	elf_pop_insert()
 #endif
@@ -235,7 +244,7 @@ extern void elf_pop_insert PARAMS ((void));
 #ifdef ANSI_PROTOTYPES
 struct ecoff_extr;
 #endif
-extern void elf_ecoff_set_ext PARAMS ((symbolS *, struct ecoff_extr *));
+extern void elf_ecoff_set_ext (symbolS *, struct ecoff_extr *);
 #endif
 
 #endif /* _OBJ_ELF_H */
